﻿using System.Diagnostics;
using System.Runtime.InteropServices;
using System;
using System.Text;

namespace Tscharlii
{
    internal class ProcessMemory
    {
        private const uint STANDARD_RIGHTS_REQUIRED = 0xF0000;
        private const uint PROCESS_VM_OPERATION     = 0x08;
        private const uint PROCESS_VM_READ          = 0x10;
        private const uint PROCESS_VM_WRITE         = 0x20;
        private const uint PAGE_EXECUTE_READWRITE   = 0x40;

        private int processHandle;
        private uint imageBase;

        public ProcessMemory()
        {
            processHandle = OpenProcess(STANDARD_RIGHTS_REQUIRED | PROCESS_VM_READ | PROCESS_VM_WRITE | PROCESS_VM_OPERATION, false, (uint)Process.GetCurrentProcess().Id);
            imageBase = 0;
        }

        ~ProcessMemory()
        {
            CloseHandle(processHandle);
        }


        public ProcessMemory(string moduleName) : this()
        {
            ProcessModuleCollection modules = Process.GetCurrentProcess().Modules;
            bool found = false;
            foreach (ProcessModule procModule in modules)
            {
                if (moduleName == procModule.ModuleName)
                {
                    imageBase = (uint)procModule.BaseAddress;
                    found = true;
                }
            }
            if (! found)
            {
                Trace.WriteLine("ProcessMemory: module " + moduleName + " not found");
            }
        }

        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern bool ReadProcessMemory(int hProcess, uint lpBaseAddress, byte[] buffer, uint size, uint lpNumberOfBytesRead);

        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern bool WriteProcessMemory(int hProcess, uint lpBaseAddress, byte[] buffer, uint size, uint lpNumberOfBytesWritten);

        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern bool VirtualProtectEx(int hProcess, uint lpAddress, uint dwSize, uint flNewProtect, out uint lpflOldProtect);

        [DllImport("kernel32.dll")]
        public static extern int OpenProcess(uint dwDesiredAccess, bool bInheritHandle, uint dwProcessId);

        [DllImport("kernel32.dll")]
        public static extern bool CloseHandle(int hHandle);

        public uint getImageBase()
        {
            return imageBase;
        }

        public bool ReadMem(uint pOffset, byte[] pBuffer, bool relative = true)
        {
            uint readFromAddress = pOffset;
            if (relative)
            {
                readFromAddress += imageBase;
            }
            if (ReadProcessMemory(processHandle, readFromAddress, pBuffer, (uint)pBuffer.LongLength, 0))
            {
                return true;
            }
            else 
            {
                Trace.WriteLine("ProcessMemory:ReadProcessMemory failed. GetLastError = " + Marshal.GetLastWin32Error());
                return false;
            }
        }

        public bool WriteMem(uint pOffset, byte[] pBuffer, bool relative = true)
        {
            uint writeToAddress = pOffset;
            if (relative)
            {
                writeToAddress += imageBase;
            }
            if (WriteProcessMemory(processHandle, writeToAddress, pBuffer, (uint)pBuffer.LongLength, 0))
            {
                return true;
            }
            else
            {
                Trace.WriteLine("ProcessMemory:WriteProcessMemory failed. GetLastError = " + Marshal.GetLastWin32Error());
                return false;
            }
        }

        public void EnableWriteAccessTo(uint pOffset, uint pSize)
        {
            uint oldProtection;
            if (!VirtualProtectEx(processHandle, imageBase + pOffset, pSize, PAGE_EXECUTE_READWRITE, out oldProtection))
            {
                Trace.WriteLine("ProcessMemory:VirtualProtectEx failed. GetLastError = " + Marshal.GetLastWin32Error());
            }
        }
    }
}
