from game import Game
from menu import *
from TheDarkWraithUserOptions import GetSilentRunning
from TheDarkWraithUserOptions import GetSubIsUnderwater
from TheDarkWraithUserOptions import GetSH5InstallPath
from TheDarkWraithUserOptions import GetHydroFollowNearestContactSweepRange
import math
import System
from System import Random
import clr
clr.AddReference( "Tscharlii" )
from Tscharlii import Sound
from Tscharlii import Hydrophone
clr.AddReference( "TDWHPT" )
from TheDarkWraith.SilentHunter5 import TDWHPT
clr.AddReference( "TDWUtils" )
from TheDarkWraith.SilentHunter5 import TDWUtils

LOGPREFIX = "Realistic Hydrophone::"

IDSTR_SONAR = 4925
IDSTR_NORMAL_SWEEP = 2885
IDSTR_NO_TARGET = 4914
IDSTR_NO_CONTACT_IN_BEARING = 4924
IDSTR_HYDRO_LOST_CONTACT = 4921
IDSTR_FOLLOWING_CONTACT_IN_BEARING = 4919
IDSTR_CLOSING = 841
IDSTR_CONSTANT_DISTANCE = 840
IDSTR_MOVING_AWAY = 842
IDSTR_CONTACT = 8804
IDSTR_CONTACT_NOW_ON = 4920


TIMER_ID_HYDROFOLLOWCONTACT = 1
TIMER_HYDROFOLLOWCONTACT_DURATION = 1.0

#SpeechModes
SM_NORMAL = "_NORMAL"
SM_WHISPER = "_WHISPER"

# Globals
initialized = False
mapControl = None
sonarman = None
random = None
rhSpeech = None
rhHydrophone = None
hydrophoneFollowContactHotkey = None
followedContact = None
lastBearingToContact = None
timer = None
aspectCalculator = None

class AspectCalculator:

	def start(self):	
		self.bearings = []
		self.lastAspect = None
		self.lastAspectReported = False
		
	def addBearing(self, newBearing):
		# the newest bearing is at the beginning of the list
		self.bearings.reverse()
		self.bearings.append(newBearing)
		self.bearings.reverse()

		index = 0
		aspect = None
		for i,bearing in enumerate(self.bearings):
			#find a bearing at least 2 degrees off the reported bearing
			bearingDelta = math.fabs(bearing - newBearing)
			if bearingDelta > 180:
				bearingDelta = 360 - bearingDelta
			if bearingDelta >= 2:
				index = i
				break
		if index != 0:
			# do we have another bearing twice as old?
			bearingsSize = len(self.bearings)
			if bearingsSize > index * 2:
				olderBearingDelta = math.fabs(self.bearings[index*2] - self.bearings[index])
				newerBearingDelta = math.fabs(self.bearings[index] - self.bearings[0])
				# normalize
				if olderBearingDelta > 180:
					olderBearingDelta = 360 - olderBearingDelta,
				if newerBearingDelta > 180:
					newerBearingDelta = 360 - newerBearingDelta
				# calculate aspect
				if newerBearingDelta > 1.05 * olderBearingDelta:
					aspect = IDSTR_CLOSING
				if newerBearingDelta < 0.95	* olderBearingDelta:
					aspect = IDSTR_MOVING_AWAY
				if newerBearingDelta > 0.975 * olderBearingDelta and newerBearingDelta < 1.025 * olderBearingDelta:
					aspect = IDSTR_CONSTANT_DISTANCE
				# drop too old obsolete bearings
				if (aspect != None) and (bearingsSize > index * 4):
					del self.bearings[index * 4::1]
					
		if aspect != None and aspect != self.lastAspect:
			self.lastAspect = aspect
			self.lastAspectReported = False

	def getAspectToReport(self):
		if not self.lastAspectReported:
			self.lastAspectReported = True
			return self.lastAspect
		return None
			
def getSonarMan():
	global sonarman
	if sonarman == None:
		sapiens = Game.SapiensManager.Sapiens
		for crewman in sapiens:
			if crewman.ObjectName == "RR_HIDRO":
				sonarman = crewman;
				return sonarman;
		ScriptManagerManaged.Trace(LOGPREFIX + "getSonarMan: Sonarman not found")	
		return None
	else:
		return sonarman
	
def getBearing( us, them ):
	contactbearing = int(math.floor(us.HeadingDEGToContact(them) - us.HeadingDEG))
	if contactbearing < 0:
		contactbearing += 360
	return contactbearing
	
def getPreciseBearing( us, them ):
	contactbearing = us.HeadingDEGToContact(them) - us.HeadingDEG
	if contactbearing < 0.0:
		contactbearing += 360.0
	return contactbearing
	
def getSpeechMode():
	# determine whether we whisper or speak normally
	# we whisper while silentrunning or if we are surfaced and have contacts
	global speechMode
	speechMode = SM_NORMAL
	if GetSilentRunning():
		speechMode = SM_WHISPER
	elif (not GetSubIsUnderwater()):
		allContacts = mapControl.AllContacts
		try:
			if allContacts.size != 0:
				speechMode = SM_WHISPER
		except:
			pass
	return speechMode	
	
def sayDigit( digit, speechMode, lastDigit ):
	# build sound id consisting of digit, speech mode and intonation
	soundId = digit.ToString( "F0" )
	soundId += speechMode
	if lastDigit:
		soundId += "_LOW"
	else:
		soundId += "_HIGH"

	# say the digit
	rhSound.playSound(soundId)
	
def sayNumber( number ):
	currentNumber = int(math.floor(number))
	numberOfDigits = math.floor( math.log10( currentNumber ) ) + 1;
	currentDigit = numberOfDigits
	speechMode = getSpeechMode()
	haveSpokenDigit = False
	while currentDigit > 0:
		# // = floored division; ** = power operator
		digit = int(currentNumber // (10 ** (currentDigit - 1)))
		# we dont say leading zeros, but we say at least one digit, even if it is a zero
		if digit > 0 or haveSpokenDigit or currentDigit == 1:
			sayDigit(digit, speechMode, (currentDigit == 1))
			haveSpokenDigit = True
			currentNumber -= (digit * (10 ** (currentDigit - 1)))
		currentDigit -= 1

def say( soundId ):
	speechMode = getSpeechMode()
	rhSound.playSound(soundId + speechMode)
	
def ReportContact(contact, changedAspect):
	
	if getSonarMan().GetIsDead():
		ScriptManagerManaged.Trace(LOGPREFIX + "ReportContact: Sonarman is dead. Cannot report contact")
		return

	if getSonarMan().GetIsWounded():
		ScriptManagerManaged.Trace(LOGPREFIX + "ReportContact: Sonarman is wounded. Cannot report contact")
		return

	if contact == None:
		ScriptManagerManaged.Trace(LOGPREFIX + "ReportContact: Not following a contact")
		return

	sub = mapControl.SubmarineContact
	bearing = getBearing(sub, contact)
	
	# report bearing + 1, if bearings get smaller
	if lastBearingToContact > bearing:
		bearing += 1
		
	# changed Aspect
	changedAspectText = ""
	if changedAspect != None:
		changedAspectText = Menu.GetLocalizedText( changedAspect )

	# report the contact textually
	msg = Menu.GetLocalizedText( IDSTR_SONAR )  + " " + Menu.GetLocalizedText( IDSTR_CONTACT_NOW_ON ) % (Menu.GetLocalizedText( IDSTR_CONTACT ), float(bearing), changedAspectText, "")
	from PageDefaultHud import Game_NewPlayerLogMessageOutside
	Game_NewPlayerLogMessageOutside( [ msg , False ] )
	
	# do not speak the contact report, if we are already talking, except we have a new aspect to report
	beQuiet = (rhSound.getNumberOfQueuedSounds() > 0) and (changedAspect == None)
	if not beQuiet:
		say("ContactNow")
		say("InBearing")
		sayNumber(bearing)
		if changedAspect == IDSTR_CLOSING:
			say("Closing")
		elif changedAspect == IDSTR_CONSTANT_DISTANCE:
			say("ConstantDistance")
		elif changedAspect == IDSTR_MOVING_AWAY:
			say("MovingAway")

def RealisticHydrophone_AnimationStopped( animation ):
	from Pagelayout import TimerID_UpdatePagelayoutGameTimer
	if animation.ID == TimerID_UpdatePagelayoutGameTimer:
		timer.Update()
		
def unableToHearContact(contact):
	# the contact can be lost, because the datastructure is freed
	contactLost = False
	try:
		contactHeading = contact.HeadingDEG
		sub = mapControl.SubmarineContact
		angle = getBearing(sub, contact)
		if angle < 10 or angle > 350 or (angle > 160 and angle < 200):
			contactLost = True
	except:
		contactLost = True
		
	# the contact will be lost, we are not below 9m
	fCurrentDepth = Menu.GuiDials.GetValue( GuiDialsWrapper.DialTypes.DEPTH_INTERNATIONAL )
	if fCurrentDepth < 9:
		contactLost = True
	
	return contactLost
		
def RealisticHydrophone_GameTimerEvent( eventID, eventSubID, elapsedTime ):

	if eventID == TIMER_ID_HYDROFOLLOWCONTACT:
		global followedContact
		global lastBearingToContact
		# Stop the timer, if the player has ordered normal sweep 
		if followedContact == None:
			timer.RemoveItem( TIMER_ID_HYDROFOLLOWCONTACT, -1 )
			return
			
		# notify the player, if the soundman has lost contact
		contactLost = unableToHearContact(followedContact) or getSonarMan().GetIsDead() or getSonarMan().GetIsWounded()
		if contactLost:
			msg = Menu.GetLocalizedText( IDSTR_SONAR ) + " " + Menu.GetLocalizedText( IDSTR_HYDRO_LOST_CONTACT ) % ("Contact",lastBearingToContact)
			from PageDefaultHud import Game_NewPlayerLogMessageOutside
			Game_NewPlayerLogMessageOutside( [ msg , False ] )
			say( "LostContactInBearing" )
			sayNumber(lastBearingToContact)
			
			timer.RemoveItem( TIMER_ID_HYDROFOLLOWCONTACT, -1 )
			followedContact = None
			lastBearingToContact = None
			return
		
		# Don't sweep back and forth near the followed contact, if the player is at the hydrophone station
		from Hydrophone import HydrophoneStation
		from UserInteractionStationsManager import UserInteractionStationsManager
		if UserInteractionStationsManager.ActiveStation == HydrophoneStation:
			aspectCalculator.start()
			return
			
		sub = mapControl.SubmarineContact
		angle = getBearing(sub, followedContact)
		
		preciseAngle = getPreciseBearing(sub, followedContact)
		aspectCalculator.addBearing(preciseAngle)
			
		# virtually turn all bearings as if angle was 180 degrees, to ease the calculation, whether the needle is before or behind the followed contact
		hydroNeedleBearing = TDWUtils.GetContactHeading(Menu.GuiDials.GetValue(GuiDialsWrapper.DialTypes.HYDROPHONE))
		normalizeDelta = 180 - angle;
		normalizedAngle = TDWUtils.GetContactHeading(angle + normalizeDelta)
		normalizedHyroNeedleMinBearing = TDWUtils.GetContactHeading(angle - GetHydroFollowNearestContactSweepRange() + normalizeDelta);
		normalizedHyroNeedleMaxBearing = TDWUtils.GetContactHeading(angle + GetHydroFollowNearestContactSweepRange() + normalizeDelta);
		normalizedHydroNeedleBearing = TDWUtils.GetContactHeading(hydroNeedleBearing + normalizeDelta);
		
		# only report contact, if hydro needle points to it and the bearing has changed
		if normalizedHydroNeedleBearing >= normalizedHyroNeedleMinBearing and normalizedHydroNeedleBearing <= normalizedHyroNeedleMaxBearing:
			if lastBearingToContact != None and lastBearingToContact != angle:
					changedAspect = aspectCalculator.getAspectToReport()
					ReportContact(followedContact, changedAspect)
			lastBearingToContact = angle
		
		# if the needle is more than HydroFollowNearestContactSweepRange degrees BEHIND the followed contact, move the needle to HydroFollowNearestContactSweepRange BEFORE the followed contact
		if normalizedHydroNeedleBearing > normalizedHyroNeedleMaxBearing:
			setHydroTo = int(TDWUtils.GetContactHeading(angle - GetHydroFollowNearestContactSweepRange()).ToString("F0"))
			if setHydroTo >= 151 and setHydroTo <= 209:
				setHydroTo = 210
			setHydrophoneNeedleToBearing(setHydroTo)
		
def setHydrophoneNeedleToBearing(angle):
	rhHydrophone.setBearing(angle)
	
def toggleHydrophoneFollowContact():
	
	global followedContact
	global lastBearingToContact
	if followedContact != None:
		followedContact = None
		lastBearingToContact = None
		msg = Menu.GetLocalizedText( IDSTR_SONAR ) + " " + Menu.GetLocalizedText( IDSTR_NORMAL_SWEEP )
		from PageDefaultHud import Game_NewPlayerLogMessageOutside
		Game_NewPlayerLogMessageOutside( [ msg , False ] )
		say("YesCaptain")
		say("NormalSweep")
		
	else:
		if getSonarMan().GetIsDead() or getSonarMan().GetIsWounded():
			return	
	
		from ObservationPeriscope import ObservationPeriscopeStation
		from AttackPeriscope import AttackPeriscopeStation
		from UZO import UZOStation
		from Hydrophone import HydrophoneStation
		from UserInteractionStationsManager import UserInteractionStationsManager
		
		activeStation = UserInteractionStationsManager.ActiveStation
	
		if activeStation == ObservationPeriscopeStation or activeStation == AttackPeriscopeStation or activeStation == UZOStation:
			if mapControl.SelectedContact != None and not unableToHearContact(mapControl.SelectedContact):
				followedContact = mapControl.SelectedContact
				sub = mapControl.SubmarineContact
				contactBearing = getBearing( sub, followedContact )
				lastBearingToContact = contactBearing
				msg = Menu.GetLocalizedText( IDSTR_SONAR ) + " " + Menu.GetLocalizedText( IDSTR_FOLLOWING_CONTACT_IN_BEARING ) % ("", float(contactBearing), "", "")
				from PageDefaultHud import Game_NewPlayerLogMessageOutside
				Game_NewPlayerLogMessageOutside( [ msg, False ] )
				say("FollowingContact")
				say("InBearing")
				sayNumber(contactBearing)
				aspectCalculator.start()
				timer.RemoveThenAddItem( TIMER_ID_HYDROFOLLOWCONTACT, -1, TIMER_HYDROFOLLOWCONTACT_DURATION, True, True )
				
			elif mapControl.SelectedContact != None:
				sub = mapControl.SubmarineContact
				contactBearing = getBearing( sub, mapControl.SelectedContact )
				msg = Menu.GetLocalizedText( IDSTR_SONAR ) + " " + Menu.GetLocalizedText( IDSTR_NO_CONTACT_IN_BEARING ) % contactBearing
				from PageDefaultHud import Game_NewPlayerLogMessageOutside
				Game_NewPlayerLogMessageOutside( [ msg, False ] )
				say("NobodyHere")	
			else:
				msg = Menu.GetLocalizedText( IDSTR_SONAR ) + " " + Menu.GetLocalizedText( IDSTR_NO_TARGET )
				from PageDefaultHud import Game_NewPlayerLogMessageOutside
				Game_NewPlayerLogMessageOutside( [ msg, False ] )
				say("NobodyHere")
		
		elif activeStation == HydrophoneStation:
			hydroangle = Menu.GuiDials.GetValue( GuiDialsWrapper.DialTypes.HYDROPHONE )
			sweeprange = GetHydroFollowNearestContactSweepRange()
			sub = mapControl.SubmarineContact
			contacts = mapControl.AllContacts

			deltaAngle = 0
			for contact in contacts:
				bearing = getBearing( sub, contact )
				if bearing >= ( hydroangle - sweeprange ) and bearing <= ( hydroangle + sweeprange ):
					if followedContact == None:
						followedContact = contact
						deltaAngle = math.fabs (hydroangle - bearing)
					else:
						if math.fabs (hydroangle - bearing) < deltaAngle:
							followedContact = contact
							deltaAngle = math.fabs (hydroangle - bearing)
							
			if unableToHearContact(followedContact):
				followedContact = None
							
			if followedContact == None:
				msg = Menu.GetLocalizedText( IDSTR_SONAR ) + " " + Menu.GetLocalizedText( IDSTR_NO_CONTACT_IN_BEARING ) % int(hydroangle.ToString( "F0" ))
				from PageDefaultHud import Game_NewPlayerLogMessageOutside
				Game_NewPlayerLogMessageOutside( [ msg, False ] )
				say("NobodyHere")
			else:
				contactBearing = getBearing( sub, followedContact )
				lastBearingToContact = contactBearing
				msg = Menu.GetLocalizedText( IDSTR_SONAR )  + " " + Menu.GetLocalizedText( IDSTR_FOLLOWING_CONTACT_IN_BEARING ) % ("", float(contactBearing), "", "")
				from PageDefaultHud import Game_NewPlayerLogMessageOutside
				Game_NewPlayerLogMessageOutside( [ msg, False ] )
				say("FollowingContact")
				say("InBearing")
				sayNumber(contactBearing)
				aspectCalculator.start()
				timer.RemoveThenAddItem( TIMER_ID_HYDROFOLLOWCONTACT, -1, TIMER_HYDROFOLLOWCONTACT_DURATION, True, True )
	
def RealisticHydrophone_KeyPressed(args):
		try:
			from menu import Utils

			if hydrophoneFollowContactHotkey[0]:
				if Utils().CheckKeyPressed(args, hydrophoneFollowContactHotkey[1], hydrophoneFollowContactHotkey[2], hydrophoneFollowContactHotkey[3], hydrophoneFollowContactHotkey[4], hydrophoneFollowContactHotkey[5]):
					toggleHydrophoneFollowContact()
		except:
			ScriptManagerManaged.Trace( "There is a problem with the hotkey mapping in RealisticHydrophoneSettings.py file" )
			
		
def initialize():
	ScriptManagerManaged.Trace(LOGPREFIX + "initialize")

	from PageDefaultHud import PageDefaultHud_MapGroup_Mapcontrol
	global mapControl
	mapControl = PageDefaultHud_MapGroup_Mapcontrol

	global sonarman
	sonarman = getSonarMan()

	global random
	random = Random()

	global rhSound
	rhSound = Sound()

	global rhHydrophone
	rhHydrophone = Hydrophone()
	
	from RealisticHydrophoneSettings import RealisticHydrophoneSettings
	RealisticHydrophoneSettings.MakeSureIsLoaded( MenuPageWrapper.InitializeScript.Now )
	from RealisticHydrophoneSettings import GetHydrophoneFollowContactHotkey
	global hydrophoneFollowContactHotkey
	hydrophoneFollowContactHotkey = GetHydrophoneFollowContactHotkey()
	
	global followedContact
	followedContact = None
	
	global lastBearingToContact
	lastBearingToContact = None
	
	global timer
	timer = TDWHPT( Menu, Game, RealisticHydrophone_GameTimerEvent )
	timer.Start()
	
	global aspectCalculator
	aspectCalculator = AspectCalculator()	
	
	from Pagelayout import Pagelayout
	Pagelayout.AnimationStopped += RealisticHydrophone_AnimationStopped
	
	from PageDefaultHud import PageDefaultHud
	PageDefaultHud.KeyPressed += RealisticHydrophone_KeyPressed
	
	# numbers speech, normal
	rhSound.addSound("0" + SM_NORMAL + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_78.wav")
	rhSound.addSound("0" + SM_NORMAL + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_79.wav")
	rhSound.addSound("1" + SM_NORMAL + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_80.wav")
	rhSound.addSound("1" + SM_NORMAL + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_81.wav")
	rhSound.addSound("2" + SM_NORMAL + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_82.wav")
	rhSound.addSound("2" + SM_NORMAL + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_83.wav")
	rhSound.addSound("3" + SM_NORMAL + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_84.wav")
	rhSound.addSound("3" + SM_NORMAL + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_85.wav")
	rhSound.addSound("4" + SM_NORMAL + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_86.wav")
	rhSound.addSound("4" + SM_NORMAL + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_87.wav")
	rhSound.addSound("5" + SM_NORMAL + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_88.wav")
	rhSound.addSound("5" + SM_NORMAL + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_89.wav")
	rhSound.addSound("6" + SM_NORMAL + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_90.wav")
	rhSound.addSound("6" + SM_NORMAL + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_91.wav")
	rhSound.addSound("7" + SM_NORMAL + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_92.wav")
	rhSound.addSound("7" + SM_NORMAL + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_93.wav")
	rhSound.addSound("8" + SM_NORMAL + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_94.wav")
	rhSound.addSound("8" + SM_NORMAL + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_95.wav")
	rhSound.addSound("9" + SM_NORMAL + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_96.wav")
	rhSound.addSound("9" + SM_NORMAL + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_97.wav")
	
	#other speech, normal
	rhSound.addSound("Closing" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_35.wav")
	rhSound.addSound("ConstantDistance" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_34.wav")
	rhSound.addSound("MovingAway" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_37.wav")
	rhSound.addSound("ContactNow" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_31.wav")
	rhSound.addSound("InBearing" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\InBearing.wav")
	rhSound.addSound("InCloseDistance" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_44.wav")
	rhSound.addSound("InMediumDistance" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_45.wav")
	rhSound.addSound("InFarDistance" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_46.wav")
	rhSound.addSound("In" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_42.wav")
	rhSound.addSound("Distance" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\Distance.wav")
	rhSound.addSound("Contact" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_28.wav")
	rhSound.addSound("TooFast" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_07.wav")
	rhSound.addSound("FollowingContact" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_01.wav")
	rhSound.addSound("LostContactInBearing" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_48.wav")
	rhSound.addSound("NobodyHere" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\NobodyHere.wav")
	rhSound.addSound("YesCaptain" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\YesCaptain.wav")
	rhSound.addSound("NormalSweep" + SM_NORMAL, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Normal\\MC_RR_HIDRO_53.wav")
	
	# numbers speech, whispering
	rhSound.addSound("0" + SM_WHISPER + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_78.wav")
	rhSound.addSound("0" + SM_WHISPER + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_79.wav")
	rhSound.addSound("1" + SM_WHISPER + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_80.wav")
	rhSound.addSound("1" + SM_WHISPER + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_81.wav")
	rhSound.addSound("2" + SM_WHISPER + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_82.wav")
	rhSound.addSound("2" + SM_WHISPER + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_83.wav")
	rhSound.addSound("3" + SM_WHISPER + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_84.wav")
	rhSound.addSound("3" + SM_WHISPER + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_85.wav")
	rhSound.addSound("4" + SM_WHISPER + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_86.wav")
	rhSound.addSound("4" + SM_WHISPER + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_87.wav")
	rhSound.addSound("5" + SM_WHISPER + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_88.wav")
	rhSound.addSound("5" + SM_WHISPER + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_89.wav")
	rhSound.addSound("6" + SM_WHISPER + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_90.wav")
	rhSound.addSound("6" + SM_WHISPER + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_91.wav")
	rhSound.addSound("7" + SM_WHISPER + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_92.wav")
	rhSound.addSound("7" + SM_WHISPER + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_93.wav")
	rhSound.addSound("8" + SM_WHISPER + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_94.wav")
	rhSound.addSound("8" + SM_WHISPER + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_95.wav")
	rhSound.addSound("9" + SM_WHISPER + "_HIGH", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_96.wav")
	rhSound.addSound("9" + SM_WHISPER + "_LOW", GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_97.wav")

	#other speech, whispering
	rhSound.addSound("Closing" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_35.wav")
	rhSound.addSound("ConstantDistance" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_34.wav")
	rhSound.addSound("MovingAway" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_37.wav")
	rhSound.addSound("ContactNow" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_31.wav")
	rhSound.addSound("InBearing" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\InBearing.wav")
	rhSound.addSound("InCloseDistance" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_44.wav")
	rhSound.addSound("InMediumDistance" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_45.wav")
	rhSound.addSound("InFarDistance" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_46.wav")
	rhSound.addSound("In" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_42.wav")
	rhSound.addSound("Distance" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\Distance.wav")
	rhSound.addSound("Contact" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_28.wav")
	rhSound.addSound("TooFast" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_07.wav")
	rhSound.addSound("FollowingContact" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_01.wav")
	rhSound.addSound("LostContactInBearing" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_48.wav")
	rhSound.addSound("NobodyHere" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\NobodyHere.wav")
	rhSound.addSound("YesCaptain" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\YesCaptain.wav")
	rhSound.addSound("NormalSweep" + SM_WHISPER, GetSH5InstallPath() + "\\data\\Sound\\Speech\\SoundMan\\Whisper\\MC_RR_HIDRO_53.wav")
	
	global initialized
	initialized = True

def destroy():
	ScriptManagerManaged.Trace(LOGPREFIX + "destroy")
	
	global rhSound
	rhSound.Dispose()
	rhSound = None

	global rhHydrophone
	rhHydrophone = None

	global mapControl
	mapControl = None

	global sonarman
	sonarman = None

	global random
	random = None
	
	global hydrophoneFollowContactHotkey
	hydrophoneFollowContactHotkey = None
	
	global followedContact
	followedContact = None
	
	global lastBearingToContact
	lastBearingToContact = None
	
	global timer
	timer.Destroy()
	timer = None
	
	global aspectCalculator
	aspectCalculator = None
	
	from PageDefaultHud import PageDefaultHud
	PageDefaultHud.KeyPressed -= RealisticHydrophone_KeyPressed
	
	from Pagelayout import Pagelayout
	Pagelayout.AnimationStopped -= RealisticHydrophone_AnimationStopped
	
	global initialized
	initialized = False

def InitializeScript():
	try:
		if not initialized:
			initialize();
	except:
		pass

def StartGame():
	if not initialized:
		initialize();

def EndGame():
	if initialized:
		destroy();

def UnloadScript():
	try:
		if initialized:
			destroy();
	except:
		pass